/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "mpdsong.h"
#include "passivepopup.h"
#include <QFont>
#include <QLocale>
#include <QStringList>

Config* Config::m_instance;

Config::Config() : QSettings("QMPDClient", "QMPDClient") {
	setObjectName("config");
}

Config *Config::instance() {
	if (!m_instance)
		m_instance = new Config();
	return m_instance;
}

/**
 * SERVER SETTINGS
 */

/// MPD Host
void Config::setServers(ServerInfoList servers) {
	QVariantList res;
	foreach(ServerInfo s, servers) {
		QVariantMap map;
		map.insert("name", s.name);
		map.insert("address", s.address);
		map.insert("port", s.port);
		map.insert("password", s.password);
		res += map;
	}
	setValue("/mpd/servers", res);
	emit serverListChanged(servers);
}

ServerInfoList Config::servers() {
	ServerInfoList res;
	QVariantList vl = value("/mpd/servers").toList();
	if (vl.isEmpty()) {
		ServerInfo si;
		si.name = "Localhost";
		si.address = "127.0.0.1";
		si.port = 6600;
		res += si;
		return res;
	}
	foreach(QVariant var, vl) {
		QVariantMap map = var.toMap();
		ServerInfo si;
		si.name = map.value("name").toString();
		si.address = map.value("address").toString();
		si.password = map.value("password").toString();
		si.port = map.value("port").toInt();
		res += si;
	}
	return res;
}

ServerInfo Config::server(QString name) {
	ServerInfoList sil = servers();
	foreach(ServerInfo si, sil) {
		if (si.name == name)
			return si;
	}
	qWarning("Server `%s' not found, returning first in list.", qPrintable(name));
	// No matches, return first server
	return sil.first();
}

void Config::setAutoconnect(bool e) {
	setValue("/mpd/autoconnect", e);
}

void Config::setReconnect(bool r) {
	setValue("/mpd/reconnect", r);
}

void Config::setReconnectTime(int t) {
	setValue("/mpd/reconnecttime", t);
}

void Config::setTimeoutTime(int t) {
	setValue("/mpd/timeout", t);
}

bool Config::autoconnect() {
	return value("/mpd/autoconnect").toBool();
}

bool Config::reconnect() {
	return value("/mpd/reconnect").toBool();
}

int Config::reconnectTime() {
	return value("/mpd/reconnecttime", 60).toInt();
}

int Config::timeoutTime() {
	return value("/mpd/timeout", 10).toInt();
}

/**
 * Look and feel settings
 */

/// Application
void Config::setEnqueue(bool e) {
	setValue("/looknfeel/enqueue", e);
}

void Config::setFont(QFont m) {
	setValue("/looknfeel/font", m);
	emit fontChanged(m);
}

void Config::setOpaqueResize(bool o) {
	setValue("/looknfeel/opaqueresize", o);
	emit opaqueResizeChanged(o);
}

void Config::setRecurse(bool r) {
	setValue("/looknfeel/recursiveview", r);
}

void Config::setScrollToPlaying(bool s) {
	setValue("/looknfeel/scrolltoplaying", s);
}

void Config::setAutoResizeColumns(bool a) {
	setValue("/looknfeel/autoresizecolumns", a);
	emit autoResizeChanged(a);
}

bool Config::enqueue() {
	return value("/looknfeel/enqueue").toBool();
}

bool Config::opaqueResize() {
	return value("/looknfeel/opaqueresize", false).toBool();
}

bool Config::recurse() {
	return value("/looknfeel/recursiveview", false).toBool();
}

bool Config::scrollToPlaying() {
	return value("/looknfeel/scrolltoplaying", false).toBool();
}

QFont Config::font() {
	return qvariant_cast<QFont>(value("/looknfeel/font"));
}

bool Config::autoResizeColumns() {
	return value("/looknfeel/autoresizecolumns", true).toBool();
}

/// Language
void Config::setTranslation(bool t) {
	setValue("/language/translate", t);
}

void Config::setLocale(QString l) {
	setValue("/language/locale", l);
}

bool Config::translation() {
	return value("/language/translate", true).toBool();
}

QString Config::locale() {
	QString tmp = value("/language/locale", "").toString();
	if (tmp.isEmpty())
		return QLocale::system().name();
	return tmp;
}

/// Notifications
void Config::setNotificationsEnabled(bool e) {
	setValue("/notifications/enabled", e);
}

void Config::setNotificationsPosition(int p) {
	setValue("/notifications/position", p);
}

void Config::setNotificationsTimeout(int t) {
	setValue("/notifications/timeout", t);
}

bool Config::notificationsEnabled() {
	return value("/notifications/enabled", true).toBool();
}

int Config::notificationsPosition() {
	return value("/notifications/position", PassivePopup::BottomRight).toInt();
}

int Config::notificationsTimeout() {
	return value("/notifications/timeout", 3).toInt();
}

/// System tray

void Config::setTrayIconEnabled(bool e) {
	setValue("/trayicon/enable", e);
	emit trayIconChanged(e);
}

void Config::setMinimizeToTray(bool e) {
	setValue("/trayicon/minimizetotray", e);
}

void Config::setStartHidden(bool e) {
	setValue("/trayicon/starthidden", e);
}

bool Config::trayIconEnabled() {
	return value("/trayicon/enable", true).toBool();
}

bool Config::minimizeToTray() {
	return value("/trayicon/minimizetotray", true).toBool();
}

bool Config::startHidden() {
	return value("/trayicon/starthidden").toBool();
}

/**
 * Tag-guesser settings
 */

void Config::setTagGuesserEnabled(bool e) {
	setValue("/tagguesser/enabled", e);
}

void Config::setGuessPattern(QString p) {
	setValue("/tagguesser/pattern", p);
}

void Config::setTestFilename(QString t) {
	setValue("/tagguesser/test", t);
}

bool Config::tagGuesserEnabled() {
	return value("/tagguesser/enabled", false).toBool();
}

QString Config::guessPattern() {
	return value("/tagguesser/pattern", "/%b/%a/%n - %t.").toString();
}

QString Config::testFilename() {
	return value("/tagguesser/test", "/ogg/Artist/Album/42 - The Title.ogg").toString();
}

/**
 * Dynamic playlist
 */
void Config::setAutoAddSongs(bool a) {
	setValue("/dynamicplaylist/add", a);
}

void Config::setAutoAddCount(int c) {
	setValue("/dynamicplaylist/count", c);
}

void Config::setAutoAddPos(int p) {
	setValue("/dynamicplaylist/pos", p);
}

void Config::setAutoRemoveSongs(bool r) {
	setValue("/dynamicplaylist/remove", r);
}

bool Config::autoAddSongs() {
	return value("/dynamicplaylist/add", false).toBool();
}

int Config::autoAddCount() {
	return value("/dynamicplaylist/count", 2).toInt();
}

int Config::autoAddPos() {
	return value("/dynamicplaylist/pos", 0).toInt();
}

bool Config::autoRemoveSongs() {
	return value("/dynamicplaylist/remove", false).toBool();
}

/**
 * Window and splitter sizes
 */

void Config::setWindowSize(QSize s) {
	setValue("/qmpdclient/windowsize", s);
}

void Config::setDirectorySplitterSizes(QByteArray l) {
	setValue("/qmpdclient/directorysplittersizes", l);
}

void Config::setLibraryHSplitterSizes(QByteArray l) {
	setValue("/qmpdclient/libraryhsplittersizes", l);
}

void Config::setLibraryVSplitterSizes(QByteArray l) {
	setValue("/qmpdclient/libraryvsplittersizes", l);
}

void Config::setMainSplitterSizes(QByteArray l) {
	setValue("/qmpdclient/mainsplittersizes", l);
}

void Config::setPlaylistsSplitterSizes(QByteArray l) {
	setValue("/qmpdclient/playlistssplittersizes", l);
}

QSize Config::windowSize() {
	return value("/qmpdclient/windowsize", QSize(800, 600)).toSize();
}

QByteArray Config::directorySplitterSizes() {
	return value("/qmpdclient/directorysplittersizes").toByteArray();
}

QByteArray Config::libraryHSplitterSizes() {
	return value("/qmpdclient/libraryhsplittersizes").toByteArray();
}

QByteArray Config::libraryVSplitterSizes() {
	return value("/qmpdclient/libraryvsplittersizes").toByteArray();
}

QByteArray Config::mainSplitterSizes() {
	return value("/qmpdclient/mainsplittersizes").toByteArray();
}

QByteArray Config::playlistsSplitterSizes() {
	return value("/qmpdclient/playlistssplittersizes").toByteArray();
}

/// Header sizes

void Config::setHeaderViewState(QString view, QByteArray ba) {
	setValue(QString("/%1/headerstate").arg(view), ba);
}

QByteArray Config::headerViewState(QString view) {
	return value(QString("/%1/headerstate").arg(view)).toByteArray();
}

/**
 * Radio stations
 */

void Config::addRadioStation(QString title, QString url) {
	QMap<QString, QVariant> map = value("/radioview/radiostations").toMap();
	map[url] = title;
	setValue("/radioview/radiostations", map);
}

void Config::removeRadioStations(MPDSongList stations) {
	QMap<QString, QVariant> map = value("/radioview/radiostations").toMap();
	foreach (MPDSong *s, stations) {
		map.remove(s->url());
	}
	setValue("/radioview/radiostations", map);
}

MPDSongList Config::radioStations() {
	QMap<QString, QVariant> map = value("/radioview/radiostations").toMap();
	QMapIterator<QString, QVariant> i(map);
	MPDSongList songs;
	while (i.hasNext()) {
		i.next();
		songs += MPDSong::createStream(i.value().toString(), i.key());
	}
	return songs;
}

