/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "controlpanel.h"
#include "mpd.h"
#include "mpdsong.h"
#include "richtext.h"
#include <QShortcut>
#include <QTime>

ControlPanel::ControlPanel(QWidget *parent) : QWidget(parent) {
	setupUi(this);
	m_mpd = MPD::instance();

	connect(prevButton, SIGNAL(clicked()), m_mpd, SLOT(prev()));
	connect(playButton, SIGNAL(clicked()), m_mpd, SLOT(play()));
	connect(pauseButton, SIGNAL(clicked()), m_mpd, SLOT(pause()));
	connect(stopButton, SIGNAL(clicked()), m_mpd, SLOT(stop()));
	connect(nextButton, SIGNAL(clicked()), m_mpd, SLOT(next()));
	connect(volumeSlider, SIGNAL(valueChanged(int)), m_mpd, SLOT(setVolume(int)));
	connect(timeSlider, SIGNAL(timeChanged(int)), m_mpd, SLOT(seek(int)));
	connect(timeSlider, SIGNAL(valueChanged(int)), this, SLOT(setElapsed(int)));
	connect(m_mpd, SIGNAL(volumeUpdated(int)), volumeSlider, SLOT(setValue(int)));
	connect(m_mpd, SIGNAL(timeUpdated(int, int)), this, SLOT(setTime(int, int)));
	connect(m_mpd, SIGNAL(timeUpdated(int, int)), timeSlider, SLOT(setTime(int, int)));
	connect(m_mpd, SIGNAL(playingSongUpdated(MPDSong *)), this, SLOT(setSong(MPDSong *)));

	// Short cuts
	QShortcut *fwd = new QShortcut(Qt::CTRL | Qt::Key_Right, this);
	QShortcut *rew = new QShortcut(Qt::CTRL | Qt::Key_Left, this);
	QShortcut *volUp = new QShortcut(Qt::CTRL | Qt::Key_Up, this);
	QShortcut *volDown = new QShortcut(Qt::CTRL | Qt::Key_Down, this);
	connect(fwd, SIGNAL(activated()), m_mpd, SLOT(seekForward()));
	connect(rew, SIGNAL(activated()), m_mpd, SLOT(seekBackward()));
	connect(volUp, SIGNAL(activated()), m_mpd, SLOT(volumeUp()));
	connect(volDown, SIGNAL(activated()), m_mpd, SLOT(volumeDown()));
	setSong(0);
}

void ControlPanel::setSong(MPDSong *s) {
	if (!s) {
		titleLabel->setText(m_mpd->isConnected() ? "" : QString("<h3>%1</h3>").arg(tr("Not connected", "qmpdclient is not connected to MPD")));
		artistLabel->setText("");
		timeSlider->setEnabled(true);
		return;
	}

	// Disable time counting for streams
	timeSlider->setEnabled(s->type() != MPDSong::PLAYLISTSTREAM);

	// Ensure labels are not too small if window is hidden
	int w = isVisible() ? titleLabel->width() : width()-200;
	QString title = formattedText("<h3>", s->title() , "</h3>", w);
	QString artist = s->artist();
	QString album = s->album();
	if (!artist.isEmpty())
		artist = formattedText("<b>", artist, "</b>", w/2);
	if (!artist.isEmpty() && !album.isEmpty())
		artist += " - ";
	if (!album.isEmpty())
		artist += formattedText("<b>", album, "</b>", w/2);

	titleLabel->setText(title);
	artistLabel->setText(artist);
}

void ControlPanel::setTime(int, int total) {
	QTime t = QTime(0, 0).addSecs(total);
	totalLabel->setText(t.toString(t.hour() > 0 ? "h:mm:ss" : "m:ss"));
}

void ControlPanel::setElapsed(int elapsed) {
	QTime t = QTime(0, 0).addSecs(elapsed);
	elapsedLabel->setText(t.toString(t.hour() > 0 ? "h:mm:ss" : "m:ss"));
}
