/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "directorymodel.h"
#include "mpddirectory.h"

DirectoryModel::DirectoryModel(QObject *parent) : QAbstractItemModel(parent), m_root(0) {
	setObjectName("directorymodel");
}
	
int DirectoryModel::columnCount(const QModelIndex &) const {
	return 1;
}

QVariant DirectoryModel::data(const QModelIndex &index, int role) const {
	MPDDirectory *i = directory(index);
	if (i && role == Qt::DisplayRole && index.column() == 0)
		return i->directoryName();
	return QVariant();
}

Qt::ItemFlags DirectoryModel::flags(const QModelIndex &) const {
	return Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled;
}

bool DirectoryModel::hasChildren(const QModelIndex &parent) const {
	// Top level
	if (!parent.isValid())
		return m_root;

	// Children
	MPDDirectory *i = directory(parent);
	if (i)
		return i->childCount() > 0;

	return false;
}

QVariant DirectoryModel::headerData(int section, Qt::Orientation orientation, int role) const {
	if (role != Qt::DisplayRole || orientation != Qt::Horizontal || section != 0)
		return QVariant();
	return QObject::tr("Directory", "This is used as the header of directory-tree in directory view");
}

QModelIndex DirectoryModel::index(int row, int column, QModelIndex const &parent) const {
	if (row < 0 || column < 0 || column > 1)
		return QModelIndex();

	// Toplevel items
	if (!parent.isValid()) {
		if (row > 0) // Only one root
			return QModelIndex();
		MPDDirectory *topLevelDirectory = m_root;
		if (topLevelDirectory)
			return createIndex(row, column, topLevelDirectory);
		return QModelIndex();
	}

	// Non-toplevel items
	MPDDirectory *parentDirectory = directory(parent);
	if (parentDirectory && row < parentDirectory->childCount()) {
		MPDDirectory *directory = parentDirectory->child(row);
		if (directory)
			return createIndex(row, column, directory);
	}
	return QModelIndex();
}

QModelIndex DirectoryModel::parent(QModelIndex const &child) const {
	MPDDirectory *childDirectory = directory(child);
	if (!childDirectory) {
		return QModelIndex();
	}

	MPDDirectory *parentDirectory = childDirectory->parent();
	if (!parentDirectory) {
		// Toplevel directory
		return QModelIndex();
	}

	MPDDirectory *parentsparent = parentDirectory->parent();
	if (parentsparent) // Find index of parent in parent's parent if any
		return createIndex(parentsparent->indexOfChild(parentDirectory), 0, parentDirectory);
	// Parent has no parent, parentDirectory is toplevel
	return createIndex(0, 0, parentDirectory);
}

int DirectoryModel::rowCount(const QModelIndex &parent) const {
	if (parent.isValid()) {
		MPDDirectory *parentDir = directory(parent);
		if (parentDir)
			return parentDir->childCount();
	}
	// Parent is not valid, requesting rowcount for toplevel dir. Always 1.
	return 1;
}

MPDDirectory *DirectoryModel::directory(const QModelIndex &index) const {
	if (!index.isValid())
		return 0;
	return static_cast<MPDDirectory*>(index.internalPointer());
}

void DirectoryModel::setDirectory(MPDDirectory *dir) {
	m_root = dir;
	reset();
}
