/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "aboutdialog.h"
#include "config.h"
#include "defines.h"
#include "directorypanel.h"
#include "librarypanel.h"
#include "mainwindow.h"
#include "mpd.h"
#include "mpdsong.h"
#include "mpdstats.h"
#include "playlistpanel.h"
#include "playlistspanel.h"
#include "preferencesdialog.h"
#include "radiopanel.h"
#include "richtext.h"
#include <QCloseEvent>
#include <QDesktopWidget>
#include <QProgressBar>
#include <QShortcut>

MainWindow::MainWindow() : QMainWindow(0) {
	setupUi(this);
	config = Config::instance();
	mpd = MPD::instance();

	setWindowTitle(NAMEVER);

	/*
	 * Status bar
	 */
	statsLabel = new QLabel();
	progressBar = new QProgressBar();
	progressBar->hide();
	progressBar->setMaximumWidth(100);
	progressBar->setMaximumHeight(statsLabel->minimumSizeHint().height());
	statusBar()->addPermanentWidget(progressBar);
	statusBar()->addPermanentWidget(statsLabel);

	// Show program name and version
	statusBar()->showMessage(NAMEVER);
	setStats(0);

	/*
	 * Ideal style vertical tabbars
	 */
	leftBar->link(leftStack, splitter);
	rightBar->link(rightStack, splitter);
	leftBar->addPanel(new PlaylistPanel(), tr("&Playlist"), QPixmap(":/icons/amarok/playlist.png"), true);
	rightBar->addPanel(new LibraryPanel(), tr("&Library"), QPixmap(":/icons/amarok/collection.png"));
	rightBar->addPanel(new DirectoryPanel(), tr("&Directories"), QPixmap(":/icons/crystal/tree.png"));
	rightBar->addPanel(new RadioPanel(), tr("&Internet Radio"), QPixmap(":/icons/crystal/stream.png"));
	rightBar->addPanel(new PlaylistsPanel(), tr("Pla&ylists"), QPixmap(":/icons/amarok/playlist.png"));

	/*
	 * Signals and slots
	 */
	connect(mpd, SIGNAL(statsUpdated(MPDStats *)), this, SLOT(setStats(MPDStats *)));
	connect(mpd, SIGNAL(playingSongUpdated(MPDSong *)), this, SLOT(setSong(MPDSong *)));
	connect(mpd, SIGNAL(disconnected(QString)), this, SLOT(disconnected(QString)));
	connect(mpd, SIGNAL(updateStart(int)), this, SLOT(updateStart(int)));
	connect(mpd, SIGNAL(updateProgress(int)), this, SLOT(updateProgress(int)));
	connect(mpd, SIGNAL(updateDone()), this, SLOT(updateDone()));

	// Menu action signals
	connect(diskonnect, SIGNAL(triggered()), mpd, SLOT(disconnectFromMPD()));
	connect(rescanLibrary, SIGNAL(triggered()), mpd, SLOT(rescanLibrary()));
	connect(aboutQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
	connect(quit, SIGNAL(triggered()), qApp, SLOT(quit()));

	// Config signals
	connect(config, SIGNAL(fontChanged(QFont)), this, SLOT(fontChanged(QFont)));
	connect(config, SIGNAL(serverListChanged(ServerInfoList)), this, SLOT(serverListChanged(ServerInfoList)));
	connect(config, SIGNAL(opaqueResizeChanged(bool)), this, SLOT(opaqueResizeChanged(bool)));

	/*
	 * Shortcut keys
	 */
	new QShortcut(Qt::Key_Escape, this, SLOT(showHide())); // Escape minimizes to tray

	/*
	 * Restore state
	 */
	serverListChanged(config->servers());
	opaqueResizeChanged(config->opaqueResize());
	splitter->restore(config->mainSplitterSizes());
	setFont(config->font());
	qApp->setFont(config->font());
	resize(config->windowSize());
}

// Auto connected slots
void MainWindow::on_about_triggered() {
	new AboutDialog(this, QPixmap(":/icons/qmpdclient22.png"), NAMEVER);
}

void MainWindow::on_splitter_splitterMoved(int, int) {
	config->setMainSplitterSizes(splitter->saveState());
}

void MainWindow::on_preferences_triggered() {
	new PreferencesDialog(this);
}

// User defined slots
void MainWindow::closeEvent(QCloseEvent *e) {
	e->accept();
	emit mainWindowClose(e);
	if (e->isAccepted())
		qApp->quit();
}

void MainWindow::connectToMPD() {
	QObject *origin = sender();
	if (!origin)
		return;
	QAction *action = qobject_cast<QAction*>(origin);
	mpd->connectToMPD(config->server(action->text()));
}

void MainWindow::disconnected(QString error) {
	if (!error.isEmpty())
		statusBar()->showMessage(QString(" %1: %2 ").arg(tr("Error")).arg(error));
	progressBar->reset();
}

void MainWindow::fontChanged(QFont font) {
	setFont(font);
	qApp->setFont(font);
}

void MainWindow::opaqueResizeChanged(bool o) {
	splitter->setOpaqueResize(o);
}

void MainWindow::resizeEvent(QResizeEvent *e) {
	config->setWindowSize(e->size());
}

void MainWindow::serverListChanged(ServerInfoList sil) {
	while (!actions.isEmpty())
		delete actions.takeLast();
	foreach(ServerInfo si, sil) {
		QAction *a = menu_Connect->addAction(QPixmap(":/icons/crystal/server.png"), si.name, this, SLOT(connectToMPD()));
		if (actions.isEmpty())
			a->setShortcut(tr("Ctrl+C"));
		actions << a;
	}
}

void MainWindow::setSong(MPDSong *s) {
	QString windowTitle = NAMEVER;
	if (s) {
		int desktopWidth = QApplication::desktop()->width();
		QString title = formattedText("", s->title(), "", desktopWidth/2);
		QString artist = formattedText("", s->artist(), "", desktopWidth/4);
		windowTitle = qApp->applicationName();
		if (!artist.isEmpty())
			windowTitle += " - " + artist;
		windowTitle += " - " + title;
	}
	setWindowTitle(windowTitle);
	setWindowIconText(windowTitle);
}

void MainWindow::setStats(MPDStats *stats) {
	if (!stats)
		stats = new MPDStats();

	unsigned long secs = stats->dbPlayTime();
	int day = secs/(60*60*24);
	secs -= day*60*60*24;
	int hour = secs/(60*60);
	secs -= hour*60*60;
	int min = secs/60;
	statsLabel->setText(QString(" %1 %2, %3 %4, %5 %6. (%7 %8 %9 %10 %11 %12) ")
	                    .arg(stats->numberOfArtists())
	                    .arg(tr("artists"))
	                    .arg(stats->numberOfAlbums())
	                    .arg(tr("albums"))
	                    .arg(stats->numberOfSongs())
	                    .arg(tr("songs"))
	                    .arg(day)
	                    .arg(tr("days"))
	                    .arg(hour)
	                    .arg(tr("hours"))
	                    .arg(min)
	                    .arg(tr("minutes")));
}

void MainWindow::showHide() {
	if (config->trayIconEnabled())
		isVisible() ? hide() : showNormal();
}

void MainWindow::updateStart(int max) {
	progressBar->setRange(0, max);
	progressBar->setValue(0);
	progressBar->show();
	statusBar()->showMessage(tr("Reading library"));
}

void MainWindow::updateProgress(int at) {
	progressBar->setValue(at);
}

void MainWindow::updateDone() {
	progressBar->setValue(progressBar->maximum());
	progressBar->hide();
	if (mpd->isConnected())
		statusBar()->showMessage(tr("Done reading library"), 2000);
	else
		statusBar()->clearMessage();
}
