/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "mpd.h"
#include "mpddirectory.h"
#include "mpdoutput.h"
#include "mpdsong.h"
#include "mpdstats.h"
#include "mpdstatus.h"
#include "tagguesser.h"
#include <QApplication>
#include <QTimer>
#include <QStringList>

MPD *MPD::m_instance;

// Constructors and such
MPD::MPD() : m_connected(false), m_connection(0), m_root(0) {
	setObjectName("mpd");
	m_config = Config::instance();
	m_instance = this;

	// Init empty stats and statuses
	m_status = new MPDStatus();
	m_prevStatus = new MPDStatus();
	m_stats = new MPDStats();
	m_prevStats = new MPDStats();
}

MPD::~MPD() {
	if (m_connected && m_connection) {
		m_mutex.lock();
		mpd_closeConnection(m_connection);
		m_mutex.unlock();
	}
	// Delete all cached items
	foreach (MPDSong *playlist, m_playlistMap.keys()) {
		MPDSongList songs = m_playlistMap.value(playlist);
		while (!songs.isEmpty())
			delete songs.takeLast();
		delete playlist;
	}
	while (!m_playlist.isEmpty())
		delete m_playlist.takeLast();
	while (!m_songs.isEmpty())
		delete m_songs.takeLast();
	while (!m_outputs.isEmpty())
		delete m_outputs.takeLast();
	delete m_root;

	// Stats/status and such
	delete m_status;
	delete m_prevStatus;
	delete m_stats;
	delete m_prevStats;
}

/**
 * Returns the one and only instace of MPD.
 * @return MPD.
 */
MPD *MPD::instance() {
	if (!m_instance)
		m_instance = new MPD();
	return m_instance;
}

/**
 * Start the MPD threads event loop.
 */
void MPD::run() {
	QTimer timer;
	connect(&timer, SIGNAL(timeout()), this, SLOT(poll()));
	timer.start(250);
	exec();
}

bool MPD::isConnected() {
	return m_connected;
}

/**
 * Connect to the MPD server.
 */
void MPD::connectToMPD(ServerInfo si) {
	if (m_connected) {
		if (m_server.port == si.port && m_server.address == si.address && m_server.password == si.password)
			return;
		// Trying to connect to another server. disconnect, then connect to it.
		disconnectFromMPD();
	}

	m_mutex.lock();
	m_connection = mpd_newConnection(si.address.toUtf8(), si.port, m_config->timeoutTime());
	if (!finishCommand()) {
		m_mutex.unlock();
		return;
	}

	if (!si.password.isEmpty()) {
		mpd_sendPasswordCommand(m_connection, si.password.toUtf8());
		if (!finishCommand()) {
			m_mutex.unlock();
			return;
		}
	}

	m_connected = true;
	m_server = si;
	m_mutex.unlock();
	emit connected(si);
}

/**
 * Disconnect from the MPD server.
 */
void MPD::disconnectFromMPD(bool lock) {
	if (lock)
		m_mutex.lock();
	if (m_connected && m_connection)
		mpd_closeConnection(m_connection);
	m_connected = false;
	m_server = ServerInfo();
	if (lock)
		m_mutex.unlock();

	emit disconnected(m_error);
	m_error = QString::null;
}

// Updates
/**
 * Recursive directory read
 */
bool MPD::readDirectory(QString dir, MPDDirectory *root, MPDDirectory *lastParent, MPDSongList *songs, int *iptr) {
	if (!m_connected)
		return false;

	QStringList dirs;
	mpd_InfoEntity *entity;
	m_mutex.lock();
	mpd_sendLsInfoCommand(m_connection, dir.toUtf8());
	while ((entity = mpd_getNextInfoEntity(m_connection))) {
		if (entity->type == MPD_INFO_ENTITY_TYPE_DIRECTORY) {
			dirs << QString::fromUtf8(entity->info.directory->path);
		} else if (entity->type == MPD_INFO_ENTITY_TYPE_SONG) {
			MPDSong *song = new MPDSong(entity);
			if (m_config->tagGuesserEnabled())
				TagGuesser::guessTags(song);
			songs->append(song);
			// Set up relations.
			m_artistAlbumMap[song->artist()].insert(song->album());
			m_artistSongMap[song->artist()].append(song);
			m_albumSongMap[song->album()].append(song);
			if (lastParent && song->url().section('/', 0, -2) == lastParent->absolutePath())
				lastParent = lastParent->addSong(song);
			else
				lastParent = root->addSong(song);
			(*iptr)++;
		}
		mpd_freeInfoEntity(entity);
	}
	if (!finishCommand()) {
		m_mutex.unlock();
		qWarning("Reading directory `%s failed", qPrintable(dir));
		return false;
	}
	m_mutex.unlock();

	// Update progress on every fiftieth song.
	if (*iptr % 50 == 0) {
		emit updateProgress(*iptr);
		/*
		 TODO when we call this, we might get signals from the GUI thread,
		which might send us off to do lots of stuff, 
		causing all kinds of strange concurrency issues.
		Have to figure out a way to avoid these recursive calls.
		*/
		QApplication::processEvents();
	}

	// Go recursive
	foreach(QString d, dirs) {
		if (!readDirectory(d, root, lastParent, songs, iptr))
			return false;
	}
	return true;
}

/**
 * Update the library, directories and playlists cache.
 */
void MPD::updateCache() {
	QString name = m_server.name;
	MPDDirectory *root = new MPDDirectory();
	MPDDirectory *lastParent = root;
	MPDSongList songs;
	m_albumSongMap.clear();
	m_artistAlbumMap.clear();
	m_artistSongMap.clear();

	if (m_connected) {
		int i=0;
		emit updateStart(m_stats->numberOfSongs());
		QApplication::processEvents();
		if (!readDirectory("/", root, lastParent, &songs, &i)) {
			if (m_connected)
				return;
			if (root)
				delete root;
			root = new MPDDirectory();
			while (!songs.isEmpty())
				delete songs.takeLast();
		}
		emit updateDone();
		QApplication::processEvents();
	}

	updatePlaylists();

	emit artistsUpdated(artists());
	emit directoriesUpdated(root);
	emit songsUpdated(songs);

	// Delete old cruft to avoid mem leaks
	if (m_root)
		delete m_root;
	m_root = root;
	while (!m_songs.isEmpty())
		delete m_songs.takeLast();
	m_songs = songs;
}

/**
 * Update playlists cache.
 */
void MPD::updatePlaylists() {
	QMap<MPDSong *, MPDSongList> playlistmap;
	MPDSongList playlists;

	if (m_connected) {
		m_mutex.lock();
		mpd_InfoEntity *entity;
		mpd_sendLsInfoCommand(m_connection, "");
		// Fetch everything in root.
		while ((entity = mpd_getNextInfoEntity(m_connection))) {
			if (entity->type == MPD_INFO_ENTITY_TYPE_PLAYLISTFILE) {
				MPDSong *tmp = new MPDSong(entity);
				playlists += tmp;
			}
			mpd_freeInfoEntity(entity);
		}
		finishCommand();
		m_mutex.unlock();

		if (m_connection->version[1] == 12) {
			foreach(MPDSong *s, playlists) {
				m_mutex.lock();
				mpd_sendListPlaylistInfoCommand(m_connection, s->url().toUtf8().data());
				while ((entity = mpd_getNextInfoEntity(m_connection))) {
					if (entity->type == MPD_INFO_ENTITY_TYPE_SONG)
						playlistmap[s] += new MPDSong(entity);
					else
						qWarning("Unknown playlist content: %d", entity->type);
					mpd_freeInfoEntity(entity);
				}
				finishCommand();
				m_mutex.unlock();
			}
		}
		else {
			foreach(MPDSong *s, playlists) {
				playlistmap[s] = MPDSongList();
			}
		}
	}

	emit playlistsUpdated(playlistmap.keys());
	foreach (MPDSong *playlist, m_playlistMap.keys()) {
		MPDSongList songs = m_playlistMap.value(playlist);
		while (!songs.isEmpty())
			delete songs.takeLast();
		delete playlist;
	}
	m_playlistMap = playlistmap;
}

/**
 * Find the song that is currently playing.
 */
void MPD::updateCurrentSong() {
	MPDSong *ret = 0;

	if (m_connected) {
		mpd_InfoEntity *entity;
		m_mutex.lock();
		mpd_sendCurrentSongCommand(m_connection);
		if ((entity = mpd_getNextInfoEntity(m_connection)) != NULL) {
			if (entity->type == MPD_INFO_ENTITY_TYPE_SONG)
				ret = new MPDSong(entity);
			mpd_freeInfoEntity(entity);
		}
		finishCommand();
		m_mutex.unlock();
	}

	emit playingSongUpdated(ret);
	if (ret)
		delete ret;
}

/**
 * Get playlist from MPD server.
 */
void MPD::updatePlaylist() {
	MPDSongList playlist;

	if (m_connected) {
		mpd_InfoEntity *entity;
		m_mutex.lock();
		mpd_sendPlaylistInfoCommand(m_connection, -1);
		while ((entity = mpd_getNextInfoEntity(m_connection))) {
			if (entity->type == MPD_INFO_ENTITY_TYPE_SONG) {
				playlist.append(new MPDSong(entity));
			}
			mpd_freeInfoEntity(entity);
		}
		finishCommand();
		m_mutex.unlock();
	}

	emit playlistUpdated(playlist);
	// Now delete old playlist songs to avoid leaks.
	while (!m_playlist.isEmpty())
		delete m_playlist.takeLast();
	m_playlist = playlist;
}

/**
 * Update the output devices
 */
void MPD::updateOutputDevices() {
	if (!m_connected)
		return;
	if (m_connection->version[1] != 12)
		return;
	while (!m_outputs.isEmpty())
		delete m_outputs.takeLast();
	mpd_OutputEntity *output;
	QMutexLocker lock(&m_mutex);
	mpd_sendOutputsCommand(m_connection);
	while ((output = mpd_getNextOutput(m_connection))) {
		m_outputs += new MPDOutput(output);
		mpd_freeOutputElement(output);
	}
	finishCommand();
}

/**
 * Update the status of the MPD server.
 */
void MPD::updateStatus() {
	delete m_prevStats;
	delete m_prevStatus;
	m_prevStats = m_stats;
	m_prevStatus = m_status;

	if (!m_connected) {
		m_stats = new MPDStats();
		m_status = new MPDStatus();
		return;
	}

	QMutexLocker lock(&m_mutex);
	// Make sure we are really connected and Stats
	mpd_sendStatsCommand(m_connection);
	mpd_Stats *stats = mpd_getStats(m_connection);
	finishCommand();
	if (!stats) {
		// An emitted signal was queued before disconnection.
		m_stats = new MPDStats();
		m_status = new MPDStatus();
		return;
	}
	m_stats = new MPDStats(stats);
	mpd_freeStats(stats);

	// Status
	mpd_sendStatusCommand(m_connection);
	mpd_Status *status = mpd_getStatus(m_connection);
	finishCommand();
	m_status = new MPDStatus(status);
	mpd_freeStatus(status);
}

// Control
/**
 * Seek to position in current playing song.
 * @param pos Position in seconds to seek to.
 */
void MPD::seek(int pos) {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendSeekCommand(m_connection, m_status->song(), pos);
	finishCommand();
}

/**
 * Seek 5 seconds backward
 */
void MPD::seekBackward() {
	if (m_status->elapsedTime() > 10) {
		m_status->setElapsedTime(m_status->elapsedTime() - 10);
		seek(m_status->elapsedTime());
	} else
		seek(0);
}

/**
 * Seek 5 seconds forward
 */
void MPD::seekForward() {
	m_status->setElapsedTime(m_status->elapsedTime() + 10);
	seek(m_status->elapsedTime());
}

/**
 * Check wheter MPD is playing or not
 * @return true if MPD is playing, else false.
 */
bool MPD::isPlaying() {
	if (!m_connected)
		return false;

	return m_status->state() == MPD_STATUS_STATE_PLAY;
}

/**
 * Check wheter MPD is paused or not.
 * @return true if MPD is paused, else false.
 */
bool MPD::isPaused() {
	if (!m_connected)
		return false;

	return m_status->state() == MPD_STATUS_STATE_PAUSE;
}

/**
 * Toggle repeat mode.
 * @param t true to enable repeat mode.
 */
void MPD::setRepeat(bool t) {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendRepeatCommand(m_connection, t);
	finishCommand();
}

/**
 * Toggle random play.
 * @param t true to enable random play
 */
void MPD::setRandom(bool t) {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendRandomCommand(m_connection, t);
	finishCommand();
}

/**
 * Set crossfade on MPD server.
 * @param secs Number of seconds to crossfade between songs, 0 to disable.
 */
void MPD::setCrossfade(int secs) {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendCrossfadeCommand(m_connection, secs);
	finishCommand();
}

/* SLOTS */

/**
 * Make MPD rescan music library
 */
void MPD::rescanLibrary() {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendUpdateCommand(m_connection, "");
	finishCommand();
}

/**
 * Stop playing.
 */
void MPD::stop() {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendStopCommand(m_connection);
	finishCommand();
}

/**
 * Pause current song.
 */
void MPD::pause() {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendPauseCommand(m_connection, !isPaused());
	finishCommand();
}

/**
 * Start playing current song.
 */
void MPD::play() {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendPlayCommand(m_connection, m_status->song());
	finishCommand();
}

/**
 * Toggle play/pause 
 */
void MPD::togglePlay() {
	if (!m_connected)
		return;
	if (isPlaying() || isPaused())
		pause();
	else
		play();
}

/**
 * Skip to next song in playlist.
 */
void MPD::next() {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendNextCommand(m_connection);
	finishCommand();
}

/**
 * Skip to previuos song in playlist
 */
void MPD::prev() {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendPrevCommand(m_connection);
	finishCommand();
}

/**
 * Set mixer volume.
 * @param v Volume from 0 to 100.
 */
void MPD::setVolume(int v) {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendSetvolCommand(m_connection, v);
	finishCommand();
}

/**
 * Increase volume by 5.
 */
void MPD::volumeUp(int up) {
	if (!m_connected)
		return;

	setVolume(m_status->volume()+qAbs(up));
}

/**
 * Decrease volume by 5.
 */
void MPD::volumeDown(int down) {
	if (!m_connected)
		return;

	setVolume(m_status->volume()-qAbs(down));
}

/**
 * Check if crossfading is enabled
 * @return Nr of seconds if enabled, 0 if not.
 */
int MPD::crossfade() {
	if (!m_connected)
		return 0;

	return m_status->crossfade();
}

QList<MPDOutput *> MPD::outputDevices() {
	return m_outputs;
}

void MPD::toggleOutputDevice(int id, bool enabled) {
	if (m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	enabled ? mpd_sendEnableOutputCommand(m_connection, id) : mpd_sendDisableOutputCommand(m_connection, id);
	finishCommand();
}

void MPD::shufflePlaylist() {
	if (!m_connected)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendShuffleCommand(m_connection);
	finishCommand();
}

void MPD::poll() {
	QApplication::processEvents();
	updateStatus();
	updateOutputDevices();

	// Simple statuses
	if (m_status->volume() != m_prevStatus->volume())
		emit volumeUpdated(m_status->volume());

	if (m_status->repeat() != m_prevStatus->repeat())
		emit repeatUpdated(m_status->repeat());

	if (m_status->random() != m_prevStatus->random())
		emit randomUpdated(m_status->random());

	if (m_status->state() != m_prevStatus->state())
		emit stateUpdated(isPlaying());

	if (m_status->totalTime() != m_prevStatus->totalTime() || m_status->elapsedTime() != m_prevStatus->elapsedTime())
		emit timeUpdated(m_status->elapsedTime(), m_status->totalTime());

	// Stats
	bool possibleSongChange = false;
	if (m_status->playlist() != m_prevStatus->playlist()) {
		updatePlaylist();

		// If songid has not changed, but playlist has, it might be a stream which has
		// switched song, since that is the way they notify song change. Update to check.
		if (m_status->songid() == m_prevStatus->songid())
			possibleSongChange = true;
	}

	if (possibleSongChange)
		updateCurrentSong();
	// Good old song change
	else if (m_status->songid() != m_prevStatus->songid())
		updateCurrentSong();
	// State change
	else if ((m_prevStatus->state() == MPD_STATUS_STATE_STOP && m_status->state() == MPD_STATUS_STATE_PLAY) ||
	         (m_prevStatus->state() == MPD_STATUS_STATE_UNKNOWN && m_status->state() == MPD_STATUS_STATE_PLAY))
		updateCurrentSong();

	if (m_stats->dbUpdateTime() != m_prevStats->dbUpdateTime()) {
		emit statsUpdated(m_stats);
		// Show playlist and playing song.
		// Must be done here to avoid letting status update, which could invalidate the if-statement
		QApplication::processEvents();
		updateCache();
	}
}

// Make sure command completes, and report any errors
bool MPD::finishCommand() {
	mpd_finishCommand(m_connection);
	if (m_connection->error) {
		QString errormsg = QString::fromUtf8(m_connection->errorStr).replace("\n", "");
		switch (m_connection->error) {
		case MPD_ERROR_TIMEOUT:
		case MPD_ERROR_SYSTEM:
		case MPD_ERROR_UNKHOST:
		case MPD_ERROR_CONNPORT:
		case MPD_ERROR_NOTMPD:
		case MPD_ERROR_NORESPONSE:
		case MPD_ERROR_SENDING:
		case MPD_ERROR_CONNCLOSED:
			// These are the serious errors. Set errormsg.
			m_error = errormsg;
			disconnectFromMPD(false);
			qWarning("Error: `%s (error code %d). Disconnecting.", qPrintable(m_error), m_connection->error);
			return false;
		default:
			// Just warnings. We ignore them.
			qWarning("Warning: `%s (error code %d). Ignoring.", qPrintable(m_error), m_connection->error);
		}
		mpd_clearError(m_connection);
		m_error = QString::null;
		return false;
	}
	// Clear error code, just in case.
	mpd_clearError(m_connection);
	m_error = QString::null;
	return true;
}
