/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MPD_H
#define MPD_H

#include <QMap>
#include <QMutex>
#include <QSet>
#include <QStringList>
#include <QThread>
#include "libmpdclient.h"
#include "mpdsonglist.h"
#include "serverinfo.h"

class Config;
class MPDDirectory;
class MPDOutput;
class MPDStats;
class MPDStatus;

class MPD : public QThread {
	Q_OBJECT

private:
	// Connection
	bool m_connected;
	Config *m_config;
	mpd_Connection *m_connection;
	ServerInfo m_server;
	QMutex m_mutex;

	// Cache
	MPDDirectory *m_root;
	MPDSongList m_songs, m_playlist;
	QMap<QString, QSet<QString> > m_artistAlbumMap;
	QMap<QString, MPDSongList> m_artistSongMap, m_albumSongMap;
	QMap<MPDSong *, MPDSongList> m_playlistMap;
	QList<MPDOutput *> m_outputs;

	// State
	QString m_error;
	MPDStatus *m_status, *m_prevStatus;
	MPDStats *m_stats, *m_prevStats;
	static MPD *m_instance;
	// Methods
	bool isPaused();
	bool isPlaying();
	bool finishCommand();
	void reverseMoveAddedSongs(int toPos);
	bool readDirectory(QString, MPDDirectory *, MPDDirectory *, MPDSongList *, int *);
	void updateCache();
	void updateCurrentSong();
	void updateOutputDevices();
	void updatePlaylist();
	void updatePlaylists();
	void updateStatus();
	MPD();

private slots:
	void poll();

public:
	~MPD();
	static MPD *instance();
	void run();
	bool isConnected();

	// Library
	QStringList albums(QString artist);
	QStringList artists();
	MPDSongList songs();
	MPDSongList songsByArtist(QString artist);
	MPDSongList songsByAlbum(QString album, QString);

	// Playlist management
	void addSongs(MPDSongList songs, bool enqueue = true, int toPos = -1);
	void addDirectories(QList<MPDDirectory *>, bool enqueue = true, int toPos = -1);
	void moveSongs(MPDSongList songs, int toPos);
	void removeSongs(MPDSongList);

	void jumpPlaylistId(int);
	void jumpPlaylistSong(MPDSong *song);

	// Playlists
	void deletePlaylists(MPDSongList playlists);
	void savePlaylist(QString playlistName);
	MPDSongList songsInPlaylists(MPDSongList);
	bool playlistExists(QString);

	// MPD control
	QList<MPDOutput *> outputDevices();
	int crossfade();

public slots:
	void clearPlaylist();
	void connectToMPD(ServerInfo);
	void disconnectFromMPD(bool = true);
	void play();
	void togglePlay();
	void pause();
	void stop();
	void next();
	void prev();
	void rescanLibrary();
	void seek(int toSeconds);
	void seekBackward();
	void seekForward();
	void setCrossfade(int seconds);
	void setRandom(bool random);
	void setRepeat(bool repeat);
	void setVolume(int v);
	void shufflePlaylist();
	void toggleOutputDevice(int, bool);
	void volumeUp(int = 5);
	void volumeDown(int = 5);

signals:
	void artistsUpdated(QStringList);
	void connected(ServerInfo);
	void disconnected(QString error);
	void directoriesUpdated(MPDDirectory *);
	void playingSongUpdated(MPDSong *);
	void playlistUpdated(MPDSongList);
	void playlistsUpdated(MPDSongList);
	void randomUpdated(bool);
	void repeatUpdated(bool);
	void songsUpdated(MPDSongList);
	void stateUpdated(bool);
	void statsUpdated(MPDStats *);
	void timeUpdated(int, int);
	void updateDone();
	void updateProgress(int);
	void updateStart(int);
	void volumeUpdated(int);
};

#endif
