/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "mpd.h"
#include "mpddirectory.h"
#include "mpdsong.h"
#include "mpdstatus.h"
#include <QStringList>

// Library management
QStringList MPD::artists() {
	return QStringList(m_artistSongMap.keys());
}

QStringList MPD::albums(QString artist) {
	QSet<QString> albums;
	if (artist.isNull()) {
		foreach (QSet<QString> a, m_artistAlbumMap.values()) {
			albums.unite(a);
		}
	} else
		albums = m_artistAlbumMap.value(artist);
	QStringList ret(albums.toList());
	ret.sort();
	return ret;
}

MPDSongList MPD::songs() {
	return m_songs;
}

MPDSongList MPD::songsByArtist(QString artist) {
	if (artist.isNull())
		return m_songs;
	return m_artistSongMap.value(artist);
}

MPDSongList MPD::songsByAlbum(QString album, QString artist) {
	if (album.isNull())
		return songsByArtist(artist);

	MPDSongList songs = m_albumSongMap.value(album);
	if (artist.isNull())
		return songs;

	// Since artist is specified, we have to make sure to get the correct
	// songs, since several artists may have the same album title
	MPDSongList ret;
	foreach (MPDSong *song, songs) {
		if (song->artist() == artist)
			ret.append(song);
	}
	return ret;
}

void MPD::addDirectories(QList<MPDDirectory*> dirs, bool enqueue, int toPos) {
	MPDSongList songs;
	foreach (MPDDirectory *dir, dirs) {
		songs << MPDSong::createTest(dir->absolutePath());
	}
	if (songs.isEmpty())
		return;
	addSongs(songs, enqueue, toPos);
	// Delete test songs to avoid memleaks
	while (!songs.isEmpty())
		delete songs.takeLast();
}

void MPD::addSongs(MPDSongList songs, bool enqueue, int toPos) {
	if (!m_connected)
		return;
	QMutexLocker lock(&m_mutex);
	mpd_sendCommandListBegin(m_connection);
	if (!enqueue)
		mpd_sendClearCommand(m_connection);
	foreach (MPDSong *song, songs) {
		if (song->type() == MPDSong::PLAYLIST)
			mpd_sendLoadCommand(m_connection, song->url().toUtf8());
		else
			mpd_sendAddCommand(m_connection, song->url().toUtf8());
	}
	if (!enqueue)
		mpd_sendPlayCommand(m_connection, 0);
	mpd_sendCommandListEnd(m_connection);
	finishCommand();
	// Finish this command, so that playlist is updated.
	reverseMoveAddedSongs(toPos);
}

void MPD::jumpPlaylistId(int id) {
	if (id < 0)
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendPlayIdCommand(m_connection, id);
	finishCommand();
}

void MPD::jumpPlaylistSong(MPDSong *song) {
	if (song)
		jumpPlaylistId(song->id());
}

void MPD::reverseMoveAddedSongs(int toPos) {
	if (!m_connected || toPos < 0)
		return;
	long long currentPlaylistId = m_status->playlist();
	QList<int> idsToMove;
	// Find new ids in playlist
	mpd_InfoEntity *entity;
	mpd_sendPlChangesCommand(m_connection, currentPlaylistId);
	while ((entity = mpd_getNextInfoEntity(m_connection))) {
		mpd_Song *song = entity->info.song;
		if (entity->type == MPD_INFO_ENTITY_TYPE_SONG && !idsToMove.contains(song->id))
			idsToMove += song->id;
		mpd_freeInfoEntity(entity);
	}
	finishCommand();
	// Move the new ids, reverse order.
	mpd_sendCommandListBegin(m_connection);
	for (int i=idsToMove.size()-1; i > -1; i--) {
		mpd_sendMoveIdCommand(m_connection, idsToMove[i], toPos);
	}
	mpd_sendCommandListEnd(m_connection);
	finishCommand();
}

void MPD::moveSongs(MPDSongList songs, int toPos) {
	if (!m_connected || songs.isEmpty())
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendCommandListBegin(m_connection);
	if (toPos < songs[0]->pos()) {
		// Moving upwards, go reverse
		for (int i=songs.size()-1; i > -1; i--) {
			mpd_sendMoveIdCommand(m_connection, songs[i]->id(), toPos);
		}
	} else {
		foreach (MPDSong *song, songs) {
			mpd_sendMoveIdCommand(m_connection, song->id(), toPos);
		}
	}
	mpd_sendCommandListEnd(m_connection);
	finishCommand();
}

// Playlists management

void MPD::deletePlaylists(MPDSongList songs) {
	if (!m_connected || songs.isEmpty())
		return;

	m_mutex.lock();
	mpd_sendCommandListBegin(m_connection);
	foreach (MPDSong *song, songs) {
		mpd_sendRmCommand(m_connection, song->url().toUtf8());
	}
	mpd_sendCommandListEnd(m_connection);
	finishCommand();
	m_mutex.unlock();

	updatePlaylists();
}

bool MPD::playlistExists(QString name) {
	foreach(MPDSong *key, m_playlistMap.keys()) {
		if (key->title() == name)
			return true;
	}
	return false;
}

void MPD::savePlaylist(QString name) {
	if (!m_connected)
		return;

	m_mutex.lock();
	mpd_sendSaveCommand(m_connection, name.toUtf8());
	finishCommand();
	m_mutex.unlock();

	updatePlaylists();
}

MPDSongList MPD::songsInPlaylists(MPDSongList pls) {
	MPDSongList ret;
	foreach (MPDSong *pl, pls) {
		foreach (MPDSong *a, m_playlistMap.value(pl)) {
			ret += a;
		}
	}
	return ret;
}

// Current playlist management

void MPD::clearPlaylist() {
	if (!m_connected)
		return;

	stop();
	QMutexLocker lock(&m_mutex);
	mpd_sendClearCommand(m_connection);
	finishCommand();
}

void MPD::removeSongs(MPDSongList songs) {
	if (!m_connected || songs.isEmpty())
		return;

	QMutexLocker lock(&m_mutex);
	mpd_sendCommandListBegin(m_connection);
	foreach (MPDSong *song, songs) {
		mpd_sendDeleteIdCommand(m_connection, song->id());
	}
	mpd_sendCommandListEnd(m_connection);
	finishCommand();
}
