/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "mpddirectory.h"
#include "mpdsong.h"
#include <QObject>

/**
 * Create a new directory
 * @param path Path of directory
 * @param abspath Absolute path of directory
 * @param parent Parent directory
 */
MPDDirectory::MPDDirectory(QString absPath, MPDDirectory *parent) :
		m_absolutePath(absPath),
		m_parent(parent),
		m_childrenMap(),
		m_songs() {
}

/**
 * One should only delete the top-most directory,
 * all child directories are automaticly deleted.
 */
MPDDirectory::~MPDDirectory() {
	foreach (QString key, m_childrenMap.keys()) {
		delete m_childrenMap.take(key);
	}
}

/**
 * Add a new song to this directory
 * @param newChild song to add
 * @return parent MPDDirectory of song.
 */
MPDDirectory *MPDDirectory::addSong(MPDSong *newSong) {
	//Get directory name of song
	QString dirName = newSong->url().section('/', 0, -2);
	if (dirName == m_absolutePath) {
		// This song belongs to me.
		m_songs += newSong;
		return this;
	}

	// Find name and path of next parent.

	// First remove my abspath from next parents, to get name of next parent
	QString nextParentsName = dirName; // Songs path
	if (!m_absolutePath.isEmpty())	// I am not root, remove my part of the path
		nextParentsName = dirName.right(dirName.size() - m_absolutePath.size() - 1);
	// Get first directory name only.
	nextParentsName = nextParentsName.section('/', 0, 0);

	// Now construct next parents absolute path
	QString nextParentsAbsPath = nextParentsName; // I am root, nextName == nextAbsPath
	if (!m_absolutePath.isEmpty()) // I am not root, add my path before next name
		nextParentsAbsPath = m_absolutePath + "/" + nextParentsName;

	MPDDirectory *newParent = 0;
	if (m_childrenMap.contains(nextParentsName)) {
		// This directory already exists.
		newParent = m_childrenMap[nextParentsName];
	} else {
		// Making new child directory
		newParent = new MPDDirectory(nextParentsAbsPath, this);
		m_childrenMap[nextParentsName] = newParent;
	}
	// Do rercursive add, until child reaches its destination.
	return newParent->addSong(newSong);
}

/**
 * Get child number x
 * @param childNumber Number of child 
 * @return Child
 */
MPDDirectory *MPDDirectory::child(int childNumber) {
	QString key = m_childrenMap.keys().at(childNumber);
	return m_childrenMap[key];
}

/**
 * Get number of children
 * @return Number of children
 */
int MPDDirectory::childCount() {
	return m_childrenMap.size();
}

/**
 * Get index of a child
 * @param child Child to find index of
 * @return Index of child
 */
int MPDDirectory::indexOfChild(MPDDirectory *child) {
	QString key = m_childrenMap.key(child);
	return m_childrenMap.keys().indexOf(key);
}

/**
 * Get parent directory of this directory
 * @return Parent directory
 */
MPDDirectory *MPDDirectory::parent() {
	return m_parent;
}

/**
 * Absolute path of directory. e.g '/Artist/Album/Disc1'
 * @return Absolute path
 */
QString MPDDirectory::absolutePath() {
	return m_absolutePath;
}

/**
 * Get name of directory. e.g. 'Disc1'
 * @return Path
 */
QString MPDDirectory::directoryName() {
	if (!m_parent)
		return QObject::tr("Root", "This is the name of the root directory in the directory browser");
	return m_absolutePath.section('/', -1);
}

/**
 * All songs in directory
 * @param recursive If true, include songs in subdriectories
 * @return Songs in directory
 */
MPDSongList MPDDirectory::songs(bool recursive) {
	if (!recursive)
		return m_songs;
	MPDSongList songs = m_songs;
	foreach (MPDDirectory *child, m_childrenMap.values()) {
		songs += child->songs(true);
	}
	return songs;
}
