/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "mpdsong.h"
#include <QTime>

MPDSong::MPDSong() :
		m_pos(0),
		m_secs(0),
		m_id(-1),
		m_type(UNKNOWN),
		m_album(QString::null),
		m_artist(QString::null),
		m_track(QString::null),
		m_date(QString::null),
		m_genre(QString::null),
		m_composer(QString::null),
		m_url(QString::null),
		m_title(QString::null),
m_time(QString::null) {}

MPDSong::MPDSong(mpd_InfoEntity *entity) : m_pos(0),
		m_secs(0),
		m_id(-1),
		m_type(UNKNOWN),
		m_album(QString::null),
		m_artist(QString::null),
		m_track(QString::null),
		m_date(QString::null),
		m_genre(QString::null),
		m_composer(QString::null),
		m_url(QString::null),
		m_title(QString::null),
m_time(QString::null) {
	if (entity->type == MPD_INFO_ENTITY_TYPE_PLAYLISTFILE)
		parsePlaylist(QString::fromUtf8(entity->info.playlistFile->path));
	else if (entity->type == MPD_INFO_ENTITY_TYPE_SONG)
		parseSong(entity->info.song);
	else
		qWarning("MPDSong::MPDSong() unknown entity type");
}

/**
 * Create new MPDSong playlist
 * @param url Url of playlist
 * @return MPDSong.
 */
MPDSong *MPDSong::createPlaylist(QString url) {
	MPDSong *song = new MPDSong();
	song->parsePlaylist(url);
	return song;
}

/**
 * Create new MPDSong stream.
 * @param title Title of stream.
 * @param url URL of stream.
 * @return MPDSong.
 */
MPDSong *MPDSong::createStream(QString title, QString url) {
	MPDSong *song = new MPDSong();
	song->m_title = title;
	song->m_url = url;
	song->m_artist = "Various artists";
	song->m_album = "Various albums";
	song->m_type = STREAM;
	return song;
}

/**
 * Create dummy MPDSong for testing tag-guesser.
 * @param url URL to guess on.
 * @return MPDSong.
 */
MPDSong *MPDSong::createTest(QString url) {
	MPDSong *song = new MPDSong();
	song->m_url = url;
	return song;
}

// Create song
/**
 * Wrap mpd_Song struct into MPDSong.
 * @param song mpd_Song from libmpdclient.
 * @return MPDSong.
 */
void MPDSong::parseSong(mpd_Song *song) {
	m_url = QString::fromUtf8(song->file);
	m_track = QString::fromUtf8(song->track);
	if (song->id > -1 && m_url.contains("://"))
		m_type = MPDSong::PLAYLISTSTREAM;
	else if (song->id > -1)
		m_type = MPDSong::PLAYLISTSONG;
	else
		m_type = MPDSong::SONG;
	m_pos = song->pos;
	m_secs = song->time;
	m_id = song->id;
	m_title = QString::fromUtf8(song->title);
	m_artist = QString::fromUtf8(song->artist);
	m_album = QString::fromUtf8(song->album);
	m_composer = QString::fromUtf8(song->composer);
	m_genre = QString::fromUtf8(song->genre);
	m_date = QString::fromUtf8(song->date);
	QTime t = QTime(0, 0).addSecs(m_secs);
	m_time = t.toString(t.hour() > 0 ? "h:mm:ss" : "m:ss");
}

// Create playlist
/**
 * Wrap mpd_InfoEntity into MPDSong.
 * @param entity mpd_InfoEntity from libmpdclient.
 * @return MPDSong.
 */
void MPDSong::parsePlaylist(QString path) {
	m_title = path;
	m_url = path;
	m_artist = "Playlist (Various artists)";
	m_album = "Playlist (Various albums)";
	m_type = MPDSong::PLAYLIST;
}

/**
 * Get hh:mm formatted time.
 * @return formatted time string.
 */
QString MPDSong::time() {
	if (m_type == STREAM || m_url.contains("://"))
		return QString::null;
	return m_time;
}

/**
 * Equality check
 * @param other MPDSong to compare this one to.
 * @return true if equal, else false.
 */
bool MPDSong::equals(MPDSong *other) {
	if (!other)
		return false;
	return (m_url == other->url()
	        && m_track == other->track()
	        && m_secs == other->secs()
	        && m_title == other->title()
			&& m_artist == other->artist()
			&& m_album == other->album()
			&& m_composer == other->composer()
			&& m_genre == other->genre()
	        && m_date == other->date()
	        && time() == other->time());
}
