/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "mpdsong.h"
#include "mpdsongmodel.h"

MPDSongModel::MPDSongModel(QObject *parent, HeaderView *hv) : QAbstractTableModel(parent),
        m_playingId(-10),
        m_view(hv->view()),
m_columns(hv->columns()) {
    setObjectName("mpdsongmodel");
}

QVariant MPDSongModel::data(const QModelIndex &index, int role) const {
    if (!index.isValid() || index.row() < 0 || index.row() >= m_visible.size() || index.column() >= m_columns.size())
        return QVariant();

    MPDSong *song = m_visible.at(index.row());
    if (!song)
        return QVariant();

    if (role == Qt::DisplayRole) {
        switch (m_columns.at(index.column())) {
        case HeaderView::TRACK:
            return song->track();
        case HeaderView::PLAYLISTPOS:
            return QString::number(song->pos()+1);
        case HeaderView::TITLE:
            return song->title();
        case HeaderView::LENGTH:
            return song->time();
        case HeaderView::ALBUM:
            return song->album();
        case HeaderView::ARTIST:
            return song->artist();
        case HeaderView::FILENAME:
            return song->filename();
        case HeaderView::URL:
            return song->url();
        case HeaderView::GENRE:
            return song->genre();
        case HeaderView::DATE:
            return song->date();
        case HeaderView::COMPOSER:
            return song->composer();
        default:
            return "?";
        }
    }

    if (role == Qt::FontRole && song->id() == m_playingId) {
        QFont font = Config::instance()->font();
        font.setBold(true);
        return font;
    }

    if (role == Qt::TextAlignmentRole && m_columns.at(index.column()) == HeaderView::LENGTH)
        return Qt::AlignRight;

    return QVariant();
}

QVariant MPDSongModel::headerData(int section, Qt::Orientation orientation, int role) const {
    if (role != Qt::DisplayRole || section >= m_columns.size() || section < 0 || orientation != Qt::Horizontal)
        return QVariant();
    HeaderView::Column col = m_columns.at(section);
    return columnName(col);
}

QString MPDSongModel::columnName(HeaderView::Column col) const {
    switch (col) {
    case HeaderView::TITLE:
        if (m_view == HeaderView::PLAYLISTSVIEW)
            return QObject::tr("Playlist");
        if (m_view == HeaderView::RADIOVIEW)
            return QObject::tr("Stream");
        return QObject::tr("Title");
    case HeaderView::TRACK:
        return QObject::tr("Track");
    case HeaderView::ARTIST:
        return QObject::tr("Artist");
    case HeaderView::ALBUM:
        return QObject::tr("Album");
    case HeaderView::LENGTH:
        return QObject::tr("Length");
    case HeaderView::URL:
        return QObject::tr("URL");
    case HeaderView::FILENAME:
        return QObject::tr("Filename");
    case HeaderView::PLAYLISTPOS:
        return "#";
    case HeaderView::GENRE:
        return QObject::tr("Genre");
    case HeaderView::DATE:
        return QObject::tr("Date");
    case HeaderView::COMPOSER:
        return QObject::tr("Composer");
    default:
        return "?";
    }
}

void MPDSongModel::setAlbumFilter(QRegExp a) {
    m_albumFilter = a;
    filter();
}

void MPDSongModel::setArtistFilter(QRegExp a) {
    m_artistFilter = a;
    filter();
}

void MPDSongModel::setFilter(QString a, QList<HeaderView::Column> fc) {
    m_filterColumns = fc;
    m_titleFilter = QRegExp(a, Qt::CaseInsensitive);
    filter();
}

int MPDSongModel::columnCount(const QModelIndex &) const {
    return m_columns.size();
}

int MPDSongModel::rowCount(const QModelIndex &) const {
    return m_visible.size();
}

Qt::ItemFlags MPDSongModel::flags(const QModelIndex &) const {
    return Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled | m_extraFlags;
}

void MPDSongModel::setExtraFlags(Qt::ItemFlags flags) {
    m_extraFlags = flags;
}

void MPDSongModel::setSongs(MPDSongList s) {
    m_all = s;
    filter();
}

MPDSong* MPDSongModel::song(const QModelIndex &index) {
    if (!index.isValid() || index.row() < 0 || m_visible.count() <= index.row())
        return 0;
    return m_visible.at(index.row());
}

MPDSongList MPDSongModel::songs() {
    return m_visible;
}

MPDSongList MPDSongModel::songs(const QModelIndex &index) {
    QModelIndexList indexes;
    indexes.append(index);
    return songs(indexes);
}

MPDSongList MPDSongModel::songs(QModelIndexList indexes) {
    MPDSongList songs;
    foreach (QModelIndex index, indexes) {
        MPDSong *s = song(index);
        if (!s || songs.contains(s))
            continue;
        songs += s;
    }
    return songs;
}

void MPDSongModel::filter() {
    if (m_titleFilter.isEmpty()
            && m_artistFilter.isEmpty()
            && m_albumFilter.isEmpty()) {
        m_visible = m_all;
    } else {
        m_visible.clear();
        foreach (MPDSong *s, m_all) {
            if (!m_artistFilter.isEmpty() && !m_artistFilter.exactMatch(s->artist()))
                continue;
            if (!m_albumFilter.isEmpty() && !m_albumFilter.exactMatch(s->album()))
                continue;

            if (m_titleFilter.isEmpty()
                    || (m_filterColumns.contains(HeaderView::PLAYLISTPOS) && QString::number(s->pos()).contains(m_titleFilter))
                    || (m_filterColumns.contains(HeaderView::TRACK) && s->track().contains(m_titleFilter))
                    || (m_filterColumns.contains(HeaderView::TITLE) && s->title().contains(m_titleFilter))
                    || (m_filterColumns.contains(HeaderView::ARTIST) && s->artist().contains(m_titleFilter))
                    || (m_filterColumns.contains(HeaderView::ALBUM) && s->album().contains(m_titleFilter))
                    || (m_filterColumns.contains(HeaderView::GENRE) && s->genre().contains(m_titleFilter))
                    || (m_filterColumns.contains(HeaderView::COMPOSER) && s->composer().contains(m_titleFilter))
                    || (m_filterColumns.contains(HeaderView::DATE) && s->date().contains(m_titleFilter))
                    || (m_filterColumns.contains(HeaderView::LENGTH) && s->time().contains(m_titleFilter))
                    || (m_filterColumns.contains(HeaderView::URL) && s->url().contains(m_titleFilter))
                    || (m_filterColumns.contains(HeaderView::FILENAME) && s->filename().contains(m_titleFilter)))
                m_visible << s;
        }
    }
    reset();
}

void MPDSongModel::sort(int column, Qt::SortOrder order) {
    if (column < 0 || column >= m_columns.size())
        return;

    m_all.sort(m_columns.at(column), order);
    filter();
}

QModelIndex MPDSongModel::indexId(int id) {
    for (int i=0; i<m_visible.size(); i++) {
        if (m_visible.at(i)->id() == id)
            return index(i, 0);
    }
    return QModelIndex();
}

MPDSong *MPDSongModel::lastSong() {
    if (m_visible.isEmpty())
        return 0;
    return m_visible.last();
}

MPDSongList MPDSongModel::unSelectedSongs(QModelIndexList selectedIndexes) {
    MPDSongList selected = songs(selectedIndexes);
    MPDSongList ret;
    foreach (MPDSong *song, m_visible) {
        if (selected.contains(song))
            continue;
        ret << song;
    }
    return ret;
}

void MPDSongModel::setPlaying(MPDSong *song) {
    if (song && m_playingId == song->id())
        return;
    QModelIndex oldidx = indexId(m_playingId);
    m_playingId = song ? song->id() : -10;
    QModelIndex idx = index(indexId(m_playingId).row(), 2);
    emit dataChanged(oldidx, idx);
}

int MPDSongModel::playingId() {
    return m_playingId;
}
