/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "metainfodialog.h"
#include "mpd.h"
#include "mpdsong.h"
#include "mpdsongmodel.h"
#include "mpdsongview.h"
#include <QContextMenuEvent>
#include <QMenu>

MPDSongView::MPDSongView(QWidget *parent, HeaderView::View v, bool makeMenu) : QTreeView(parent),
		m_midButtonDrag(false),
		m_view(v),
		m_config(Config::instance()),
		m_mpd(MPD::instance()),
		m_menu(new QMenu(this)),
		m_enqueueAction(0),
		m_playAction(0),
m_informationAction(0) {
	m_header = new HeaderView(this, m_view, false);
	setModel(m_model = new MPDSongModel(this, m_header));
	setObjectName(m_header->objectName());
	setHeader(m_header);
	m_header->restore();

	connect(this, SIGNAL(activated(const QModelIndex &)), this, SLOT(indexActivated(const QModelIndex &)));
	if (makeMenu) {
		m_enqueueAction = m_menu->addAction(QPixmap(":/icons/add.png"), tr("&Enqueue"), this, SLOT(enqueue()));
		m_playAction = m_menu->addAction(QPixmap(":/icons/play.png"), tr("&Play"), this, SLOT(play()));
		m_menu->addSeparator();
		m_informationAction = m_menu->addAction(QPixmap(":/icons/crystal/info.png"), tr("&Information..."), this, SLOT(information()));
	}

	selectionChanged();
}

void MPDSongView::contextMenuEvent(QContextMenuEvent *e) {
	QModelIndex clicked = indexAt(e->pos());
	QModelIndexList indexes = selectedIndexes();
	if (indexes.isEmpty() || !indexes.contains(clicked))
		setCurrentIndex(indexAt(e->pos()));
	m_menu->popup(e->globalPos());
}

void MPDSongView::indexActivated(const QModelIndex &index) {
	m_mpd->addSongs(m_model->songs(index), m_config->enqueue());
}

void MPDSongView::selectionChanged(const QItemSelection &s, const QItemSelection &u) {
	QAbstractItemView::selectionChanged(s, u);
	bool t = selectedIndexes().size() > 0;
	if (m_enqueueAction)
		m_enqueueAction->setEnabled(t);
	if (m_playAction)
		m_playAction->setEnabled(t);
	if (m_informationAction)
		m_informationAction->setEnabled(t);
}

void MPDSongView::setSongs(MPDSongList songs) {
	m_model->setSongs(songs);
	if (m_header->isClickable())
		m_model->sort(m_header->sortIndicatorSection(), m_header->sortIndicatorOrder());
	selectionChanged();
}

void MPDSongView::information() {
	new MetaInfoDialog(this, m_model->songs(selectedIndexes()), m_view == HeaderView::RADIOVIEW);
}

void MPDSongView::enqueue() {
	m_mpd->addSongs(m_model->songs(selectedIndexes()));
}

void MPDSongView::play() {
	m_mpd->addSongs(m_model->songs(selectedIndexes()), false);
}

void MPDSongView::setAlbumFilter(QRegExp needle) {
	m_model->setAlbumFilter(needle);
	selectionChanged();
}

void MPDSongView::setArtistFilter(QRegExp needle) {
	m_model->setArtistFilter(needle);
	selectionChanged();
}

void MPDSongView::setTitleFilter(const QString &needle) {
	m_model->setFilter(needle, m_header->visibleColumns());
	selectionChanged();
}

void MPDSongView::startDrag(Qt::DropActions) {
	if (m_midButtonDrag)
		return;
	QDrag *drag = new QDrag(this);
	if (m_view == HeaderView::RADIOVIEW)
		drag->setPixmap(QPixmap(":/icons/crystal/stream.png"));
	else
		drag->setPixmap(QPixmap(":/icons/amarok/collection.png"));
	QString songs = "";
	foreach (MPDSong *song, m_model->songs(selectedIndexes())) {
		songs += song->url() + "\n";
	}
	if (songs.isEmpty())
		return;
	QMimeData *mimeData = new QMimeData();
	mimeData->setData("qmpdclient/song", songs.toUtf8());
	drag->setMimeData(mimeData);
	drag->start();
}

void MPDSongView::mouseMoveEvent(QMouseEvent *e) {
	QMouseEvent *e2 = e;
	if (m_midButtonDrag)
		e2 = new QMouseEvent(e->type(), e->pos(), Qt::LeftButton, e->buttons() | Qt::LeftButton, e->modifiers());
	QTreeView::mouseMoveEvent(e2);
}

void MPDSongView::mousePressEvent(QMouseEvent *e) {
	QMouseEvent *e2 = e;
	if (e->button() == Qt::MidButton) {
		m_midButtonDrag = true;
		e2 = new QMouseEvent(e->type(), e->pos(), Qt::LeftButton, e->buttons(), e->modifiers());
	}
	QTreeView::mousePressEvent(e2);
}

void MPDSongView::mouseReleaseEvent(QMouseEvent *e) {
	if (e->button() == Qt::MidButton)
		m_midButtonDrag = false;
	QTreeView::mouseReleaseEvent(e);
}
