/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "mpd.h"
#include "mpdoutput.h"
#include "mpdsong.h"
#include "mpdsongmodel.h"
#include "passivepopup.h"
#include "preferencesdialog.h"
#include "servermodel.h"
#include "tagguesser.h"
#include <QDir>
#include <QFontDialog>

PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
	setupUi(this);
	config = Config::instance();
	mpd = MPD::instance();
	setWindowTitle(tr("Preferences"));
	setAttribute(Qt::WA_DeleteOnClose, true);

	splitter->setStretchFactor(0, 1);
	splitter->setStretchFactor(1, 10);

	// Connection
	serverList->setModel(serverModel = new ServerModel(this));
	serverSel = serverList->selectionModel();
	connect(serverSel, SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection &)),
	        this, SLOT(serverSelectionChanged()));
	serverModel->setServers(config->servers());
	deleteButton->setEnabled(false);
	upButton->setEnabled(false);
	downButton->setEnabled(false);
	autoConnectCheck->setChecked(config->autoconnect());
	timeoutSpin->setValue(config->timeoutTime());
	reconnectCheck->setChecked(config->reconnect());
	reconnectSpin->setValue(config->reconnectTime());

	// Server settings
	crossfadeCheck->setChecked(mpd->crossfade() > 0);
	crossfadeSpin->setValue(mpd->crossfade());
	foreach (MPDOutput *o, mpd->outputDevices()) {
		QTreeWidgetItem *i = new QTreeWidgetItem(outputList, o->id());
		i->setText(1, o->name());
		i->setCheckState(0, o->enabled() ? Qt::Checked : Qt::Unchecked);
	}

	// Tray icon
	trayIconCheck->setChecked(config->trayIconEnabled());
	startHiddenCheck->setChecked(config->startHidden());
	minToTrayCheck->setChecked(config->minimizeToTray());

	// Notifications
	PassivePopup::Position pos = (PassivePopup::Position)config->notificationsPosition();
	positionGroup = new QButtonGroup(this);
	positionGroup->addButton(topLeft, PassivePopup::TopLeft);
	positionGroup->addButton(top, PassivePopup::Top);
	positionGroup->addButton(topRight, PassivePopup::TopRight);
	positionGroup->addButton(right, PassivePopup::Right);
	positionGroup->addButton(bottomRight, PassivePopup::BottomRight);
	positionGroup->addButton(bottom, PassivePopup::Bottom);
	positionGroup->addButton(bottomLeft, PassivePopup::BottomLeft);
	positionGroup->addButton(left, PassivePopup::Left);
	switch (pos) {
	case PassivePopup::TopLeft: topLeft->setChecked(true); break;
	case PassivePopup::Top: top->setChecked(true); break;
	case PassivePopup::TopRight: topRight->setChecked(true); break;
	case PassivePopup::Right: right->setChecked(true); break;
	case PassivePopup::Bottom: bottom->setChecked(true); break;
	case PassivePopup::BottomLeft: bottomLeft->setChecked(true); break;
	case PassivePopup::Left: left->setChecked(true); break;
	default: bottomRight->setChecked(true); break;
	}
	notificationsTimeoutSpinner->setValue(config->notificationsTimeout());
	notificationsCheck->setChecked(config->notificationsEnabled());

	// Look and feel
	setFontString(font());
	if (config->enqueue())
		enqueueItemRadio->setChecked(true);
	else
		playItemRadio->setChecked(true);
	autoResizeCheck->setChecked(config->autoResizeColumns());
	centerPlayingCheck->setChecked(config->scrollToPlaying());
	recursiveViewCheck->setChecked(config->recurse());
	opaqueCheck->setChecked(config->opaqueResize());

	// Language
	translationCheck->setChecked(config->translation());
	// Find all translations
	QDir dir(":/translations/");
	dir.setFilter(QDir::Files);
	QFileInfoList list = dir.entryInfoList();
	for (int i = 0; i < list.size(); ++i) {
		QFileInfo fileInfo = list.at(i);
		localeList->addItem(fileInfo.baseName().section("_", 1, 1));
	}

	QList<QListWidgetItem *>foundLocales = localeList->findItems(config->locale(), Qt::MatchExactly);
	if (foundLocales.isEmpty())
		localeList->setCurrentRow(0);
	else
		localeList->setCurrentItem(foundLocales.first());

	// Tagguesser
	tagGuesserCheck->setChecked(config->tagGuesserEnabled());
	patternLine->setText(config->guessPattern());
	testLine->setText(config->testFilename());
	origPattern = config->guessPattern();

	// Auto player
	autoAddCheck->setChecked(config->autoAddSongs());
	autoAddCountSpin->setValue(config->autoAddCount());
	autoAddPosSpin->setValue(config->autoAddPos());
	autoRemoveCheck->setChecked(config->autoRemoveSongs());

	// Views
	categoryList->header()->hide();
	widgetStack->setCurrentIndex(0);
	categoryList->setCurrentItem(categoryList->topLevelItem(0));

#ifndef Q_WS_X11
	// No win32 trayicon support
	widgetStack->removeWidget(trayIconPage);
	QList<QTreeWidgetItem *> items = categoryList->findItems(tr("Tray icon"), Qt::MatchExactly, 0);
	if (!items.isEmpty())
		categoryList->takeTopLevelItem(categoryList->indexOfTopLevelItem(items[0]));
#endif
	show();

	for (int i=0; i<serverModel->columnCount(); i++)
		serverList->resizeColumnToContents(i);
}

void PreferencesDialog::on_okButton_clicked() {
	// Server
	config->setServers(serverModel->servers());

	config->setTimeoutTime(timeoutSpin->value());
	config->setAutoconnect(autoConnectCheck->isChecked());
	config->setReconnect(reconnectCheck->isChecked());
	config->setReconnectTime(reconnectSpin->value());

	// Server settings
	if (crossfadeSpin->value() > 0 && crossfadeCheck->isChecked())
		mpd->setCrossfade(crossfadeSpin->value());
	else
		mpd->setCrossfade(0);
	foreach (QTreeWidgetItem *i, outputList->findItems("*", Qt::MatchWildcard)) {
		mpd->toggleOutputDevice(i->type(), i->checkState(0) == Qt::Checked ? true : false);
	}

	// Look and feel
	config->setEnqueue(enqueueItemRadio->isChecked());
	config->setFont(fontLine->font());
	config->setAutoResizeColumns(autoResizeCheck->isChecked());
	config->setOpaqueResize(opaqueCheck->isChecked());
	config->setScrollToPlaying(centerPlayingCheck->isChecked());
	config->setRecurse(recursiveViewCheck->isChecked());

	// Language
	config->setTranslation(translationCheck->isChecked());
	QList<QListWidgetItem*> locales = localeList->selectedItems();
	if (locales.isEmpty() || locales.first() == localeList->item(0))
		config->setLocale("");
	else
		config->setLocale(locales.first()->text());

	// Tag guesser
	config->setTagGuesserEnabled(tagGuesserCheck->isChecked());
	config->setGuessPattern(patternLine->text());
	config->setTestFilename(testLine->text());

	// Trayicon
	config->setTrayIconEnabled(trayIconCheck->isChecked());
	config->setMinimizeToTray(minToTrayCheck->isChecked());
	config->setStartHidden(startHiddenCheck->isChecked());

	// Auto player
	config->setAutoAddSongs(autoAddCheck->isChecked());
	config->setAutoAddPos(autoAddPosSpin->value());
	config->setAutoAddCount(autoAddCountSpin->value());
	config->setAutoRemoveSongs(autoRemoveCheck->isChecked());

	// Notifications
	config->setNotificationsPosition(positionGroup->checkedId());
	config->setNotificationsTimeout(notificationsTimeoutSpinner->value());
	config->setNotificationsEnabled(notificationsCheck->isChecked());

	close();
}

void PreferencesDialog::on_cancelButton_clicked() {
	// Revert to original guess pattern
	config->setGuessPattern(origPattern);
	close();
}

void PreferencesDialog::on_fontButton_clicked() {
	bool ok;
	QFont newfont = QFontDialog::getFont(&ok, fontButton->font(), this);
	if (ok) {
		setFontString(newfont);
		emit resizeEvent(0);
	}
}

void PreferencesDialog::setFontString(QFont newfont) {
	QString font = newfont.family();
	if (newfont.bold())
		font += QString(", %1").arg(tr("bold"));
	if (newfont.italic())
		font += QString(", %1").arg(tr("italic"));
	font += QString(", %1").arg(newfont.pointSize());
	fontLine->setText(font);
	fontLine->setFont(newfont);
}

void PreferencesDialog::on_testLine_textChanged(const QString &url) {
	if (url.isEmpty())
		return;
	// Create song to guess on.
	MPDSong *song = new MPDSong();
	song->setUrl(url);
	TagGuesser::guessTags(song);
	bandLabel->setText(QString("%1: %2").arg(tr("Artist")).arg(song->artist()));
	albumLabel->setText(QString("%1: %2").arg(tr("Album")).arg(song->album()));
	trackLabel->setText(QString("%1: %2").arg(tr("Track")).arg(song->track()));
	titleLabel->setText(QString("%1: %2").arg(tr("Title")).arg(song->title()));
	delete song;
}

void PreferencesDialog::on_patternLine_textChanged(const QString &pattern) {
	config->setGuessPattern(pattern);
	if (pattern.isEmpty())
		return;
	on_testLine_textChanged(testLine->text());
}

void PreferencesDialog::on_categoryList_currentItemChanged(QTreeWidgetItem *c, QTreeWidgetItem *) {
	int index = categoryList->indexOfTopLevelItem(c);
	widgetStack->setCurrentIndex(index);
	currentLabel->setText("<h3>" + c->text(0) + "</h3>");
}

void PreferencesDialog::on_addButton_clicked() {
	ServerInfoList sil = serverModel->servers();
	ServerInfo si;
	si.name = "Edit me";
	si.address = "127.0.0.1";
	si.port = 6600;
	sil << si;
	serverModel->setServers(sil);
}

void PreferencesDialog::on_deleteButton_clicked() {
	foreach (QModelIndex sel, serverSel->selectedIndexes()) {
		serverModel->deleteServer(sel);
		break;
	}
	deleteButton->setEnabled(serverModel->size() > 1 && !serverSel->selectedIndexes().isEmpty());
}

void PreferencesDialog::serverSelectionChanged() {
	deleteButton->setEnabled(serverModel->size() > 1 && !serverSel->selectedIndexes().isEmpty());
	foreach (QModelIndex sel, serverSel->selectedIndexes()) {
		upButton->setEnabled(sel.row() > 0);
		downButton->setEnabled(sel.row() < serverModel->size()-1);
		return;
	}
	upButton->setEnabled(false);
	downButton->setEnabled(false);
}

void PreferencesDialog::on_downButton_clicked() {
	foreach (QModelIndex sel, serverSel->selectedIndexes()) {
		if (serverModel->moveDown(sel)) {
			serverSel->setCurrentIndex(serverModel->index(sel.row()+1, 0),
			                           QItemSelectionModel::Clear);
			serverSel->setCurrentIndex(serverModel->index(sel.row()+1, 0),
			                           QItemSelectionModel::Select | QItemSelectionModel::Rows);
		}
		return;
	}
}

void PreferencesDialog::on_upButton_clicked() {
	foreach (QModelIndex sel, serverSel->selectedIndexes()) {
		if (serverModel->moveUp(sel)) {
			serverSel->setCurrentIndex(serverModel->index(sel.row()-1, 0),
			                           QItemSelectionModel::Clear);
			serverSel->setCurrentIndex(serverModel->index(sel.row()-1, 0),
			                           QItemSelectionModel::Select | QItemSelectionModel::Rows);
		}
		return;
	}
}
