/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "trayicon.h"
#include "trayiconlabel.h"
#include <QCloseEvent>

#ifdef Q_WS_X11
#include <QX11Info>
Atom TrayIcon::systemTraySelection = None;
Window TrayIcon::systemTrayWindow = None;
QCoreApplication::EventFilter TrayIcon::oldEventFilter = 0;
TrayIconLabel *TrayIcon::trayIcon = 0;
#endif
TrayIcon *TrayIcon::m_instance = 0;
bool TrayIcon::installed = false;

/**
 * This class manages the tray icon that sits in Freedesktop compliant system trays.
 */
TrayIcon::TrayIcon() {
	setObjectName("trayicon");
	config = Config::instance();

#ifdef Q_WS_X11
	// Install event filter, so we can install our selves in the system tray
	// if it is reomved and then becomes re-available.
	if (!oldEventFilter) {
		oldEventFilter = qApp->setEventFilter(eventFilter);
		Window root = QX11Info::appRootWindow();
		Display *dpy = QX11Info::display();
		XWindowAttributes attr;
		XGetWindowAttributes(dpy, root, &attr);
		if ((attr.your_event_mask & StructureNotifyMask) != StructureNotifyMask)
			XSelectInput(dpy, root, attr.your_event_mask | StructureNotifyMask);
	}
	
	connect(config, SIGNAL(trayIconChanged(bool)), this, SLOT(trayIconChanged(bool)));
#endif
}

TrayIcon *TrayIcon::instance() {
	if (!m_instance) {
		m_instance = new TrayIcon();
		m_instance->trayIconChanged(Config::instance()->trayIconEnabled());
	}
	return m_instance;
}

TrayIcon::~TrayIcon() {
	remove();
}

bool TrayIcon::isInstalled() {
	return installed;
}

void TrayIcon::install() {
#ifdef Q_WS_X11
	installed = true;
	locateSystemTray();
	installTrayIcon();
#endif
}

void TrayIcon::remove() {
#ifdef Q_WS_X11
	installed = false;
	if (systemTrayWindow != None) // Stop notifications
		XSelectInput(QX11Info::display(), systemTrayWindow, 0);
	systemTrayWindow = None;
	installTrayIcon();
#endif
}

void TrayIcon::locateSystemTray() {
#ifdef Q_WS_X11
	if (!TrayIcon::installed) {
		systemTrayWindow = None;
		return;
	}
	Display *dpy = QX11Info::display();
	// Find the system tray's Window
	QString szAtom = QString("_NET_SYSTEM_TRAY_S%1").arg(QX11Info::appScreen());
	systemTraySelection = XInternAtom(dpy, szAtom.toLatin1(), false);
	systemTrayWindow = XGetSelectionOwner(dpy, systemTraySelection);
#endif
}

void TrayIcon::installTrayIcon() {
#ifdef Q_WS_X11
	if (!TrayIcon::installed || systemTrayWindow == None) {
		if (trayIcon) {
			trayIcon->hide();
			delete trayIcon;
		}
		trayIcon = 0;
		return;
	}

	trayIcon = new TrayIconLabel();
	QObject::connect(trayIcon, SIGNAL(showHide()), TrayIcon::instance(), SIGNAL(showHide()));

	Display *dpy = QX11Info::display();
	// We want to be notified of events regarding system tray
	XSelectInput(dpy, systemTrayWindow, StructureNotifyMask);

	XEvent ev;
	memset(&ev, 0, sizeof(ev));
	ev.xclient.type = ClientMessage;
	ev.xclient.window = systemTrayWindow;
	ev.xclient.message_type = XInternAtom(dpy, "_NET_SYSTEM_TRAY_OPCODE", False);
	ev.xclient.format = 32;
	ev.xclient.data.l[0] = CurrentTime;
	ev.xclient.data.l[1] = 0; //SYSTEM_TRAY_REQUEST_DOCK
	ev.xclient.data.l[2] = trayIcon->winId();
	ev.xclient.data.l[3] = 0;
	ev.xclient.data.l[4] = 0;
	XSendEvent(dpy, systemTrayWindow, False, NoEventMask, &ev);
#endif
}

bool TrayIcon::eventFilter(void *message, long *result) {
	bool retval = false;
#ifdef Q_WS_X11
	if (TrayIcon::oldEventFilter)
		retval = TrayIcon::oldEventFilter(message, result);

	if (!TrayIcon::installed)
		return retval;

	Display *display = QX11Info::display();
	XAnyEvent *ev = (XAnyEvent *)message;
	if  (ev->type == DestroyNotify && ev->window == systemTrayWindow) {
		locateSystemTray();
		installTrayIcon(); // add it to the new tray
		retval = true;
	} else if (ev->type == ClientMessage && systemTrayWindow == None) {
		static Atom manager_atom = XInternAtom(display, "MANAGER", False);
		XClientMessageEvent *cm = (XClientMessageEvent *)message;
		if ((cm->message_type == manager_atom) && ((Atom)cm->data.l[1] == systemTraySelection)) {
			systemTrayWindow = cm->data.l[2];
			installTrayIcon();
		}
		retval = true;
	}
#endif
	return retval;
}

void TrayIcon::mainWindowClose(QCloseEvent *e) {
	if (!hideOnClose())
		return;
	e->ignore();
	emit showHide();
}

bool TrayIcon::hideOnClose() {
	return isInstalled() && config->trayIconEnabled() && config->minimizeToTray();
}

bool TrayIcon::hideOnStart() {
	return isInstalled() && config->trayIconEnabled() && config->startHidden();
}

void TrayIcon::trayIconChanged(bool enabled) {
	if (enabled && !isInstalled())
		install();
	else if (!enabled && isInstalled())
		remove();
}
