/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "defines.h"
#include "mpd.h"
#include "mpdsong.h"
#include "richtext.h"
#include "trayiconlabel.h"

#include <QApplication>
#include <QDesktopWidget>
#include <QMenu>
#include <QMouseEvent>
#ifdef Q_WS_X11
#include <QX11Info>
#include <X11/Xlib.h>
#endif

/*
 * TODO: Inherit background from system tray.
 */
TrayIconLabel::TrayIconLabel() : QLabel(0, Qt::FramelessWindowHint | Qt::X11BypassWindowManagerHint) {
	setObjectName("traylabel");
	setMinimumSize(22, 22);
	setMaximumSize(22, 22);

	/*
	* Inherit background from system tray.
	* TODO: This does not work very well. The icon degrades
	* in quality when you move the mouse pointer over it.
	*/
	/*	setAutoFillBackground(false);
	setAttribute(Qt::WA_NoSystemBackground);
	setAttribute(Qt::WA_Hover);
	setAttribute(Qt::WA_PaintOnScreen);*/

	// Reference to MPD
	mpd = MPD::instance();

	// Make menu please
	menu = new QMenu(this);
	menu->addAction(QPixmap(":/icons/next.png"), tr("Next"), mpd, SLOT(next()));
	menu->addAction(QPixmap(":/icons/play.png"), tr("Play"), mpd, SLOT(play()));
	menu->addAction(QPixmap(":/icons/pause.png"), tr("Pause"), mpd, SLOT(pause()));
	menu->addAction(QPixmap(":/icons/stop.png"), tr("Stop"), mpd, SLOT(stop()));
	menu->addAction(QPixmap(":/icons/prev.png"), tr("Previuos"), mpd, SLOT(prev()));
	menu->addSeparator();
	menu->addAction(QPixmap(":/icons/crystal/exit.png"), tr("Quit"), qApp, SLOT(quit()));

	connect(mpd, SIGNAL(playingSongUpdated(MPDSong*)), this, SLOT(setSong(MPDSong*)));
	connect(mpd, SIGNAL(connected(ServerInfo)), this, SLOT(connected()));
	connect(mpd, SIGNAL(disconnected(QString)), this, SLOT(disconnected()));

	if (mpd->isConnected())
		setPixmap(QPixmap(":/icons/qmpdclient22.png"));
	else
		setPixmap(QPixmap(":/icons/qmpdclient-dimmed.png"));

	setToolTip(NAMEVER);
}

/**
 * Catch the enter event. Needed to show tooltip.
 */
void TrayIconLabel::enterEvent(QEvent *e) {
#ifdef Q_WS_X11
	XEvent ev;
	memset(&ev, 0, sizeof(ev));
	ev.xfocus.display = QX11Info::display();
	ev.xfocus.type = FocusIn;
	ev.xfocus.window = winId();
	ev.xfocus.mode = NotifyNormal;
	ev.xfocus.detail = NotifyAncestor;
	qApp->x11ProcessEvent(&ev);
#endif
	QWidget::enterEvent(e);
}

/**
 * Catch the leave event. Needed to terminate tooltip correctly.
 */
void TrayIconLabel::leaveEvent(QEvent *e) {
#ifdef Q_WS_X11
	XEvent ev;
	memset(&ev, 0, sizeof(ev));
	ev.xfocus.display = QX11Info::display();
	ev.xfocus.type = FocusOut;
	ev.xfocus.window = winId();
	ev.xfocus.mode = NotifyNormal;
	ev.xfocus.detail = NotifyAncestor;
	qApp->x11ProcessEvent(&ev);
#endif
	QWidget::leaveEvent(e);
}

void TrayIconLabel::wheelEvent(QWheelEvent *e) {
	int numDegrees = e->delta() / 8;
	int numSteps = numDegrees / 15;
	if (e->orientation() == Qt::Vertical) {
		if (numSteps > 0)
			mpd->volumeUp(5*numSteps);
		else if (numSteps < 0)
			mpd->volumeDown(5*numSteps);
		e->accept();
	}
	else
		e->ignore();
}

/**
 * If left button is clicked, notify those who are interested.
 * If right button is clicked, popup menu.
 */
void TrayIconLabel::mousePressEvent(QMouseEvent *e) {
	if (rect().contains(e->globalPos()))
		return;

	if (e->button() == Qt::LeftButton)
		emit showHide();
	else if (e->button() == Qt::RightButton && menu)
		menu->popup(e->globalPos());
}

#ifdef Q_WS_X11
bool TrayIconLabel::x11Event(XEvent *ev) {
	if (ev->type == ReparentNotify)
		show();
	return QWidget::x11Event(ev);
}
#endif

void TrayIconLabel::setSong(MPDSong *s) {
	if (!s) {
		// Got NULL song
		QString tooltip = "<html><body>";
		tooltip += "<table><tr><td><img src=':/icons/qmpdclient16.png'></td><td valign='middle'>";
		tooltip += QString("&nbsp;%1</td></tr></table>").arg(NAMEVER);
		tooltip += QString("<b>%1</b>").arg(tr("Not playing", "This is for the trayicon tooltip, indicating that no song is playing"));
		tooltip += "</body></html>";
		setToolTip(tooltip);
		return;
	}

	// Make nice tooltip
	int desktopWidth = QApplication::desktop()->width();
	QString artist = formattedText("", s->artist(), "", desktopWidth/8);
	QString album = formattedText("", s->album(), "", desktopWidth/8);
	QString tooltip = "<html><body>";
	tooltip += "<table><tr><td><img src=':/icons/qmpdclient16.png'></td><td valign='middle'>";
	tooltip += QString("&nbsp;%1</td></tr></table>").arg(NAMEVER);
	tooltip += formattedText("<b>", s->title(), "</b>", desktopWidth/4) + "<br />";
	if (!artist.isEmpty())
		tooltip += artist;
	if (!artist.isEmpty() && !album.isEmpty())
		tooltip += " - ";
	if (!album.isEmpty())
		tooltip += album;
	tooltip += "<br />";
	tooltip += "</body></html>";
	setToolTip(tooltip);
	return;
}

void TrayIconLabel::connected() {
	setPixmap(QPixmap(":/icons/qmpdclient22.png"));
}

void TrayIconLabel::disconnected() {
	setPixmap(QPixmap(":/icons/qmpdclient-dimmed.png"));
}
